from fastapi import APIRouter, Depends, HTTPException, UploadFile, File, Form
from sqlmodel import Session
from datetime import datetime
from backend.models.models import Product, ProductImage, User
from backend.schemas.product import ProductRead, ProductImageRead
from backend.database import get_session
from backend.routes.auth import get_current_user
import shutil
import os

router = APIRouter()

static_dir = os.path.join("app", "static")
os.makedirs(static_dir, exist_ok=True)


@router.post("/", response_model=ProductRead)
def create_product(
    name: str = Form(...),
    description: str = Form(None),
    price: float = Form(...),
    stock: int = Form(...),
    shop_id: int = Form(...),
    category_id: int = Form(None),
    images: UploadFile = File(None),  # Ensuring image is correctly set
    session: Session = Depends(get_session),
):
    product = Product(
        name=name,
        description=description,
        price=price,
        stock=stock,
        shop_id=shop_id,
        category_id=category_id,
        created_at=datetime.utcnow(),
    )
    session.add(product)
    session.commit()
    session.refresh(product)

    # Handling image upload
    if images and images.filename:
        file_ext = os.path.splitext(images.filename)[1]  # Get file extension
        file_name = f"product_{product.id}{file_ext}"
        file_location = os.path.join(static_dir, file_name)

        with open(file_location, "wb") as buffer:
            shutil.copyfileobj(images.file, buffer)

        # Save image record in ProductImage
        product_image = ProductImage(product_id=product.id, image_url=file_location)
        session.add(product_image)
        session.commit()

    return product


@router.get("/{product_id}", response_model=ProductRead)
def read_product(product_id: int, session: Session = Depends(get_session)):
    product = session.get(Product, product_id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    return product


@router.put("/{product_id}", response_model=ProductRead)
def update_product(
    product_id: int,
    name: str = Form(None),
    description: str = Form(None),
    price: float = Form(None),
    stock: int = Form(None),
    category_id: int = Form(None),
    file: UploadFile = File(None),
    session: Session = Depends(get_session),
):
    product = session.get(Product, product_id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    
    if name:
        product.name = name
    if description:
        product.description = description
    if price is not None:
        product.price = price
    if stock is not None:
        product.stock = stock
    if category_id is not None:
        product.category_id = category_id

    session.add(product)
    session.commit()
    session.refresh(product)

    if file and file.filename:
        file_location = os.path.join(static_dir, f"product_{product.id}_{file.filename}")
        with open(file_location, "wb") as buffer:
            shutil.copyfileobj(file.file, buffer)
        
        image = ProductImage(product_id=product.id, image_url=file_location)
        session.add(image)
        session.commit()

    return product


@router.delete("/{product_id}")
def delete_product(product_id: int, session: Session = Depends(get_session)):
    product = session.get(Product, product_id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    session.delete(product)
    session.commit()
    return {"message": "Product deleted successfully"}


@router.post("/ProductImage/", response_model=ProductImageRead)
def upload_product_image(
    product_id: int = Form(...),
    file: UploadFile = File(...),
    session: Session = Depends(get_session),
):
    product = session.get(Product, product_id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")

    file_location = os.path.join(static_dir, f"product_{product.id}_{file.filename}")
    with open(file_location, "wb") as buffer:
        shutil.copyfileobj(file.file, buffer)

    image = ProductImage(product_id=product.id, image_url=file_location)
    session.add(image)
    session.commit()
    return image


@router.get("/ProductImage/{product_id}", response_model=list[ProductImageRead])
def get_product_images(product_id: int, session: Session = Depends(get_session)):
    images = session.query(ProductImage).filter(ProductImage.product_id == product_id).all()
    if not images:
        raise HTTPException(status_code=404, detail="No images found for this product")
    return images


@router.delete("/ProductImage/{image_id}")
def delete_product_image(image_id: int, session: Session = Depends(get_session)):
    image = session.get(ProductImage, image_id)
    if not image:
        raise HTTPException(status_code=404, detail="Image not found")
    session.delete(image)
    session.commit()
    return {"message": "Product image deleted successfully"}
